<?php

namespace EventPortal\Tests\Feature\Delegate;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Tests\TestCase;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Schema;
use Illuminate\Validation\ValidationException;
use Spatie\Snapshots\MatchesSnapshots;

class ResetPasswordTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    public function setUp(): void
    {
        parent::setUp();

        Schema::table('users', static function (Blueprint $table): void {
            if (!Schema::hasColumn('users', 'password')) {
                $table->string('password')->nullable();
            }
        });
    }

    /** @test **/
    public function a_delegate_can_reset_their_password()
    {
        $oldPassword = 'old-password';
        $newPassword = 'new-password';

        $delegate = factory(Delegate::class)
            ->state('snapshot')
            ->create();
        $delegate->user->password = Hash::make($oldPassword);
        $delegate->user->save();

        $this->actingAs($delegate->user)
            ->fromRoute('account.index')
            ->followingRedirects()
            ->post(route('account.password.update'), [
                'current_password' => $oldPassword,
                'password' => $newPassword,
                'password_confirmation' => $newPassword,
            ])->assertOk();

        $delegate->refresh();
        $this->assertTrue(Hash::check($newPassword, $delegate->user->password));
    }

    /** @test **/
    public function a_delegate_cannot_reset_their_password_if_it_does_not_meet_requirements()
    {
        config(['auth-password.validation.password' => [
            'min:4',
        ]]);

        $oldPassword = 'old-password';
        $newPassword = 'new';

        $delegate = factory(Delegate::class)
            ->state('snapshot')
            ->create();
        $delegate->user->password = Hash::make($oldPassword);
        $delegate->user->save();

        try {
            $this->actingAs($delegate->user)
                ->withoutExceptionHandling()
                ->post(route('account.password.update'), [
                    'current_password' => $oldPassword,
                    'password' => $newPassword,
                    'password_confirmation' => $newPassword,
                ]);
        } catch (ValidationException $exception) {
            $this->assertArrayNotHasKey('current_password', $exception->errors());
            $this->assertArrayHasKey('password', $exception->errors());
            $this->assertTrue(Hash::check($oldPassword, $delegate->user->password));

            return;
        }

        $this->fail('Password validation exception was not thrown.');
    }

    /** @test **/
    public function a_delegate_cannot_reset_their_password_if_their_old_one_is_incorrect()
    {
        $oldPassword = 'old-password';
        $newPassword = 'new';

        $delegate = factory(Delegate::class)
            ->state('snapshot')
            ->create();
        $delegate->user->password = Hash::make($oldPassword);
        $delegate->user->save();

        try {
            $this->actingAs($delegate->user)
                ->withoutExceptionHandling()
                ->post(route('account.password.update'), [
                    'current_password' => 'WRONG PASSWORD',
                    'password' => $newPassword,
                    'password_confirmation' => $newPassword,
                ]);
        } catch (ValidationException $exception) {
            $this->assertArrayHasKey('current_password', $exception->errors());
            $this->assertTrue(Hash::check($oldPassword, $delegate->user->password));

            return;
        }

        $this->fail('Password validation exception was not thrown.');
    }
}
